/*
	MarkovGen - A Markov Chain Program
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>

#include "markovchain.hpp" //Why does this break if put after arguments?
#include "arguments.hpp"
#include "util.hpp"

namespace
{
	void Main(const Arguments &arguments)
	{
		File input(stdin);
		MarkovChain chain(arguments.max_match, arguments.max_generate);
		try {
			while (true){
				std::string s;
				while (true){
					//Read a character
					char c = 0;
					input.Read(&c, 1);
					
					//Check that it's not a filtered character
					if (arguments.allow.size() && !arguments.allow.count(c) && !arguments.split.count(c)){
						continue;
					}
					if (arguments.filter.count(c) && !arguments.split.count(c)){
						continue;
					}
					
					//Split the string appropriately
					if (arguments.split.count(c)){
						break;
					}
					s += c;
					if (!arguments.split.size()){
						break;
					}
				}
				
				chain.Record(s);
			}
		}
		catch (File::EndOfFile e){}
		
		File output(stdout);
		while (true){
			std::vector<std::string> ss = chain.Generate(arguments.min_match, arguments.max_match, arguments.min_generate, arguments.max_generate);
			for (auto it = ss.rbegin(); it != ss.rend(); it++){
				std::cout << *it << arguments.post_char;
			}
			std::cout << arguments.post_group;
		}
	}
}

int main(int argc, char **argv)
{
	try {
		Arguments arguments(argc, argv);
		Main(arguments);
	}
	catch (Error e) {
		e.OutputMessage();
		return 1;
	}
	return 0;
}
